@@ -28,7 +28,7 @@ is_infix <- function(vi) {
   return(FALSE)
 }
 
-#' Build a (non-empty) data.frame.
+#' Build a data.frame from the user's description.
 #'
 #' A convenient way to build a data.frame in legible transposed form.  Position of
 #' first "|" (or other infix operator) determines number of columns
@@ -64,11 +64,7 @@ build_frame <- function(..., cf_eval_environment = parent.frame()) {
   lv <- length(v)
   # inspect input
   if(lv<1) {
-    stop("wrapr::build_frame expect at least a header, one column, and one row")
-  }
-  cls <- vapply(v, class, character(1))
-  if(sum(cls=="call") < 1) {
-    stop("wrapr::build_frame expected at least 1 infix operator")
+    return(data.frame())
   }
   # unpack
   unpack_val <- function(vi) {
@@ -124,6 +120,9 @@ build_frame <- function(..., cf_eval_environment = parent.frame()) {
   vu <- lapply(v, unpack_val)
   vu <- Reduce(c, lapply(vu, as.list))
   ncol <- length(vu)
+  if(ncol<1) {
+    stop("wrapr::build_frame() zero columns")
+  }
   is_name <- vapply(vu, is.name, logical(1))
   if(any(is_name)) {
     ncol <- which(is_name)[[1]]-1
@@ -133,18 +132,29 @@ build_frame <- function(..., cf_eval_environment = parent.frame()) {
   if(abs(nrow - round(nrow))>0.1) {
     stop("wrapr::build_frame confused as to cell count")
   }
-  seq <- seq_len(nrow)*ncol
-  fr <- data.frame(x = unlist(vu[seq + 1],
-                              recursive = FALSE,
-                              use.names = FALSE),
-                   stringsAsFactors = FALSE)
-  colnames(fr) <- as.character(vu[[1]])
-  if(ncol>1) {
-    for(i in 2:ncol) {
-      ci <- as.character(vu[[i]])
-      fr[[ci]] <-  unlist(vu[seq + i],
-                          recursive = FALSE,
-                          use.names = FALSE)
+  if(nrow<=0) {
+    fr <- data.frame(x = logical(0))
+    colnames(fr) <- as.character(vu[[1]])
+    if(ncol>1) {
+      for(i in 2:ncol) {
+        ci <- as.character(vu[[i]])
+        fr[[ci]] <- logical(0)
+      }
+    }
+  } else {
+    seq <- seq_len(nrow)*ncol
+    fr <- data.frame(x = unlist(vu[seq + 1],
+                                recursive = FALSE,
+                                use.names = FALSE),
+                     stringsAsFactors = FALSE)
+    colnames(fr) <- as.character(vu[[1]])
+    if(ncol>1) {
+      for(i in 2:ncol) {
+        ci <- as.character(vu[[i]])
+        fr[[ci]] <-  unlist(vu[seq + i],
+                            recursive = FALSE,
+                            use.names = FALSE)
+      }
     }
   }
   rownames(fr) <- NULL
@@ -152,13 +162,13 @@ build_frame <- function(..., cf_eval_environment = parent.frame()) {
 }
 
 
-#' Render a data.frame in build_frame format.
+#' Render a simple data.frame in build_frame format.
 #'
-#' @param x data.frame (atomic types, with at least 1 row and 1 column).
+#' @param x data.frame (with atomic types).
 #' @param ... not used for values, forces later arguments to bind by name.
 #' @param time_format character, format for "POSIXt" classes.
 #' @param formatC_options named list, options for formatC()- used on numerics.
-#' @return chracter
+#' @return character
 #'
 #' @seealso \code{\link{build_frame}},  \code{\link{qchar_frame}}
 #'
@@ -200,16 +210,12 @@ draw_frame <- function(x,
   if(!is.data.frame(x)) {
     stop("draw_frame x needs to be a data.frame")
   }
+  res <- "wrapr::build_frame()"
   nrow <- nrow(x)
-  if(nrow<1) {
-    stop("draw_frame x needs at least 1 row")
-  }
   ncol <- ncol(x)
-  if(ncol<1) {
-    stop("draw_frame x needs at least 1 column")
+  if((nrow>=1) && (ncol<1)) {
+    stop("wrapr::draw_frame bad input: no columns, but has rows")
   }
-  # convert to character matrix
-  xq <- x
   qts <- function(v) {
     # wayts to quote: dput(), shQuote(), deparse()
     vapply(as.character(v),
@@ -218,69 +224,79 @@ draw_frame <- function(x,
            },
            character(1))
   }
-  for(ci in colnames(x)) {
-    if("POSIXt" %in% class(x[[ci]])) {
-      xq[[ci]] <- paste0("\"",
-                         format(x[[ci]], time_format),
-                         "\"")
-    } else if(is.character(x[[ci]]) || is.factor(x[[ci]])) {
-      xq[[ci]] <- qts(as.character(x[[ci]]))
-    } else if(is.integer(x[[ci]])) {
-      xq[[ci]] <- paste0(format(x[[ci]], scientific = FALSE), "L")
-    } else if(is.numeric(x[[ci]])) {
-      xq[[ci]] <- formatC(x[[ci]],
-                          digits = formatC_args$digits,
-                          width =  formatC_args$width,
-                          format =  formatC_args$format,
-                          flag =  formatC_args$flag,
-                          mode =  formatC_args$mode,
-                          big.mark =  formatC_args$big.mark,
-                          big.interval =  formatC_args$big.interval,
-                          small.mark =  formatC_args$small.mark,
-                          small.interval =  formatC_args$small.interval,
-                          decimal.mark =  formatC_args$decimal.mark,
-                          preserve.width =  formatC_args$preserve.width,
-                          zero.print =  formatC_args$zero.print,
-                          drop0trailing =  formatC_args$drop0trailing)
-    } else {
-      xq[[ci]] <- as.character(x[[ci]])
+  if((nrow<1) || (ncol<1)) {
+    if(ncol>=1) {
+      res <- paste(qts(colnames(x)), collapse = ", ")
+      res <- paste0("wrapr::build_frame(", res, ")")
     }
-    xq[[ci]][is.na(x[[ci]])] <- NA
+  } else {
+    # convert to character matrix
+    xq <- x
+
+    for(ci in colnames(x)) {
+      if("POSIXt" %in% class(x[[ci]])) {
+        xq[[ci]] <- paste0("\"",
+                           format(x[[ci]], time_format),
+                           "\"")
+      } else if(is.character(x[[ci]]) || is.factor(x[[ci]])) {
+        xq[[ci]] <- qts(as.character(x[[ci]]))
+      } else if(is.integer(x[[ci]])) {
+        xq[[ci]] <- paste0(format(x[[ci]], scientific = FALSE), "L")
+      } else if(is.numeric(x[[ci]])) {
+        xq[[ci]] <- formatC(x[[ci]],
+                            digits = formatC_args$digits,
+                            width =  formatC_args$width,
+                            format =  formatC_args$format,
+                            flag =  formatC_args$flag,
+                            mode =  formatC_args$mode,
+                            big.mark =  formatC_args$big.mark,
+                            big.interval =  formatC_args$big.interval,
+                            small.mark =  formatC_args$small.mark,
+                            small.interval =  formatC_args$small.interval,
+                            decimal.mark =  formatC_args$decimal.mark,
+                            preserve.width =  formatC_args$preserve.width,
+                            zero.print =  formatC_args$zero.print,
+                            drop0trailing =  formatC_args$drop0trailing)
+      } else {
+        xq[[ci]] <- as.character(x[[ci]])
+      }
+      xq[[ci]][is.na(x[[ci]])] <- NA
+    }
+    xm <- as.matrix(xq)
+    xm <- matrix(data = as.character(xm),
+                 nrow = nrow, ncol = ncol)
+    # convert header to values
+    xm <- rbind(matrix(data = qts(colnames(x)),
+                       nrow = 1, ncol = ncol),
+                xm)
+    # compute padding
+    widths <- nchar(xm)
+    widths[is.na(as.numeric(widths))] <- 2
+    colmaxes <- matrix(data = apply(widths, 2, max),
+                       nrow = nrow+1, ncol = ncol,
+                       byrow = TRUE)
+    padlens <- colmaxes - widths
+    pads <- matrix(data = vapply(padlens,
+                                 function(vi) {
+                                   paste(rep(' ', vi), collapse = '')
+                                 }, character(1)),
+                   nrow = nrow+1, ncol = ncol)
+    # get intermediates
+    seps <- matrix(data = ", ",
+                   nrow = nrow+1, ncol = ncol)
+    seps[, ncol] <- " |"
+    seps[nrow+1, ncol] <- " )"
+    # format
+    fmt <- matrix(data = paste0(xm, pads, seps),
+                  nrow = nrow+1, ncol = ncol)
+    rlist <- vapply(seq_len(nrow+1),
+                    function(i) {
+                      paste(fmt[i, , drop=TRUE], collapse = '')
+                    }, character(1))
+    rlist <- paste0("   ", rlist)
+    res <- paste(rlist, collapse = "\n")
+    res <- paste0("wrapr::build_frame(\n", res, "\n")
   }
-  xm <- as.matrix(xq)
-  xm <- matrix(data = as.character(xm),
-               nrow = nrow, ncol = ncol)
-  # convert header to values
-  xm <- rbind(matrix(data = qts(colnames(x)),
-                     nrow = 1, ncol = ncol),
-              xm)
-  # compute padding
-  widths <- nchar(xm)
-  widths[is.na(as.numeric(widths))] <- 2
-  colmaxes <- matrix(data = apply(widths, 2, max),
-                     nrow = nrow+1, ncol = ncol,
-                     byrow = TRUE)
-  padlens <- colmaxes - widths
-  pads <- matrix(data = vapply(padlens,
-                               function(vi) {
-                                 paste(rep(' ', vi), collapse = '')
-                               }, character(1)),
-                 nrow = nrow+1, ncol = ncol)
-  # get intermediates
-  seps <- matrix(data = ", ",
-                 nrow = nrow+1, ncol = ncol)
-  seps[, ncol] <- " |"
-  seps[nrow+1, ncol] <- " )"
-  # format
-  fmt <- matrix(data = paste0(xm, pads, seps),
-                nrow = nrow+1, ncol = ncol)
-  rlist <- vapply(seq_len(nrow+1),
-                  function(i) {
-                    paste(fmt[i, , drop=TRUE], collapse = '')
-                  }, character(1))
-  rlist <- paste0("   ", rlist)
-  res <- paste(rlist, collapse = "\n")
-  res <- paste0("wrapr::build_frame(\n", res, "\n")
   if(is.name(x_s)) {
     res <- paste0(as.character(x_s), " <- ", res)
   }
@@ -289,11 +305,11 @@ draw_frame <- function(x,
 
 
 
-#' Build a (non-empty) quoted data.frame.
+#' Build a quoted data.frame.
 #'
 #' A convenient way to build a character data.frame in legible transposed form.  Position of
 #' first "|" (or other infix operator) determines number of columns
-#' (all other infic operators are aliases for ",").
+#' (all other infix operators are aliases for ",").
 #' Names are treated as character types.
 #'
 #' @param ... cell names, first infix operator denotes end of header row of column names.
@@ -322,16 +338,26 @@ qchar_frame <- function(...) {
   v <- as.list(substitute(list(...))[-1])
   lv <- length(v)
   env <- parent.frame()
-  # inspect input
   if(lv<1) {
-    stop("wrapr::qchar_frame expect at least a header, one column, and one row")
+    return(data.frame())
   }
+  # inspect input
   cls <- vapply(v, class, character(1))
   if(length(setdiff(cls, c("character", "call", "name")))>0) {
     stop("wrapr::qchar_frame expect only strings, names, +, and commas")
   }
   if(sum(cls=="call") < 1) {
-    stop("wrapr::qchar_frame expected at least 1 infix operator")
+    # no rows case
+    fr <- data.frame(x = character(0),
+                     stringsAsFactors = FALSE)
+    colnames(fr) <- as.character(v[[1]])
+    if(lv>1) {
+      for(i in 2:lv) {
+        fr[[as.character(v[[i]])]] <- character(0)
+      }
+    }
+    rownames(fr) <- NULL
+    return(fr)
   }
   ncol <- match("call", cls)
   # unpack
